import java.util.Enumeration;
import java.applet.*;
import java.net.*;

/*
 * Autor: Martin Kot
 * Nazev: Castice
 * Studijni cislo: KOT119
 */

/**
 * Hlavni trida expertniho systemu. Tridu je mozno uzit jako applet i aplikaci.
 *
 * @author Martin Kot
 * @version 1.00
 */
public class Castice extends Applet{
  /**Tabulka pravidel z baze znalosti*/
  private TabulkaPravidel tabulkaPravidel;
  /**Tabulka cilu a dotazu z baze znalosti*/
  private TabulkaListu tabulkaListu;
  /**Odkaz na uzivatelske rozhrani*/
  private UIExpert ui;

  /**
   * Konstruktor
   *
   * @param uiExpert Odkaz na uzivatelske rozhrani
   * @param filename Jmeno souboru s bazi znalosti
   */
  public Castice(UIExpert uiExpert, String filename) {
    this.tabulkaPravidel = new TabulkaPravidel();
    this.tabulkaListu = new TabulkaListu();
    this.ui=uiExpert;
    (new SyntAnal(uiExpert,filename,tabulkaPravidel,tabulkaListu)).analyzuj();
  }
  
  /** 
   * Konstruktor bez parametru slouzici pro vytvoreni instance v appletu
   */  
  public Castice() {
    this.tabulkaPravidel = new TabulkaPravidel();
    this.tabulkaListu = new TabulkaListu();
    this.ui=new GUIExpert(true,this);
  }   
  
  /**
   * Provadi pseudodopredne retezeni
   */
  public void dopredne() {
    Enumeration enumer = tabulkaPravidel.getRules();
    int pocet_cilu=0;
    while(enumer.hasMoreElements()){
      Pravidlo p = (Pravidlo) enumer.nextElement();
      int ph = p.getPodm().hodnota().getVal();
      if (ph == PravdHod.TRUE) {
        UzelList cil = p.getCil();
        cil.setVal(true);
        if(cil.getTyp()==UzelList.CIL){
	  pocet_cilu++;
          ui.cil(cil.getText());
        }
      }
      else if (ph == PravdHod.FALSE) {
        UzelList cil = p.getCil();
        cil.setVal(false);
      }
      else {
        UzelList cil = p.getCil();
        cil.setReseno(true);
        if(cil.getTyp()==UzelList.CIL){
          ui.cilUnknown(cil.getText());
        }
      }
    }
    if(pocet_cilu == 0) {
      ui.nenasel();
    }
  }

  /**
   * Provede zpetne retezeni
   */
  public void zpetne() {
    Enumeration enumer;
    enumer = tabulkaListu.getCile();
    int pocet_cilu=0;
    while(enumer.hasMoreElements()){
      UzelList cil = (UzelList) enumer.nextElement();
      int ph = cil.hodnota().getVal();
      if (ph == PravdHod.TRUE) {
        cil.setVal(true);
	pocet_cilu++;
        ui.cil(cil.getText());
      }
      else if (ph == PravdHod.FALSE) {
        cil.setVal(false);
      }
      else {
        cil.setReseno(true);
        ui.cilUnknown(cil.getText());
      }
    }
    if(pocet_cilu == 0) {
      ui.nenasel();
    }
  }

  /**
   * Inicializacni metoda appletu. Nacte z HTML stranky jmeno souboru a spusti 
   * provadeni zpetneho retezeni.
   */
  public void init(){
    String filename = getParameter("fileName");
    URL url = getCodeBase();
    (new SyntAnal(ui,url, filename,tabulkaPravidel,
           tabulkaListu)).analyzuj();
    hledejZpet();
  }
  
  /**
   * Provadi dopredne retezeni a po skonceni se dotaze na dalsi pokracovani
   */
  public void hledejDop() {
    boolean pokrac =true;
    while (pokrac) {
      this.dopredne();
      pokrac=ui.pokracovat();
      if(pokrac){
        tabulkaPravidel.znovu();
        ui.znovu();
      }
    }
  }

  /**
   * Provadi zpetne retezeni a po skonceni se dotaze na dalsi pokracovani
   */
  public void hledejZpet() {
    boolean pokrac =true;
    while (pokrac) {
      this.zpetne();
      pokrac=ui.pokracovat();
      if(pokrac){
        tabulkaPravidel.znovu();
        ui.znovu();
      }
    }
  }
  
  /**
   * Vytiskne na standardni vystup strucnou napovedu
   */
  private static void help(){
    System.out.println("\nProgram slouzi k rozpoznavani fyzikalnich castic podle jejich vlastnosti.");  
    System.out.println("\nSpusteni programu: java Castice [argumenty]");  
    System.out.println("Argument: \"-t\"  - program pobezi v textovem rezimu. Jinak je zvolen textovy");
    System.out.println("                  rezim, neni-li mozno spustit graficky");
    System.out.println("          \"-d\"  - je zvoleno dopredne retezeni");
    System.out.println("          \"-h\"  - vypise tuto napovedu a konci");
    System.out.println("          \"...\" - prvni jiny argument, nez vyjmenovane, bude bran jako");
   System.out.println("                  jmeno souboru s bazi znalosti\n");
   System.out.println("Program muze bezet i jako applet. V tomto pripade je nutno zadat do HTML");
   System.out.println("stranky jako do tagu PARAM atribut se jmenem \"fileName\" a hodnotou jmeno");
   System.out.println("souboru s bazi znalosti.\n\n");
  }

  /**
   * Spousteci metoda aplikace
   */
  public static void main(String[] argv){
    boolean pokrac =true;
    UIExpert uiExp;
    String filename = null;
    boolean dopredne = false;
    boolean textove = false;
    //Parsing argumentu prikazove radky
    if (argv.length != 0) {
        for (int i = 0; i < argv.length; i++) {
            String arg = argv[i];
            if (arg.startsWith("-")) {
                if (arg.equals("-d")) {
                    dopredne = true;
                    continue;
                }
                if (arg.equals("-t")) {
                    textove=true;
                    continue;
                }
                if (arg.equals("-h")) {
                    help();
                    System.exit(0);
                    continue;
                }
                if (arg.equals("-h")) {
                    System.exit(1);
                }
            }
	    else {
              if(filename == null){
                filename=arg;
              }
            }
        }
    }
    else {
      help();
      System.exit(0);
    }

   
  if(dopredne){
      if(textove){
        Castice  castice = new Castice(new TextUIExpert(), argv[0]);
        castice.hledejDop();
      }
      else {
        try {
          Castice castice = new Castice(new GUIExpert(), argv[0]);
          castice.hledejDop();
        } catch(java.lang.InternalError e) {
          //Nepodarilo se spustit graficky, zkusi textove
          Castice  castice = new Castice(new TextUIExpert(), argv[0]);
          castice.hledejDop();
        }
      }
    }
    else {
      if(textove){
        Castice  castice = new Castice(new TextUIExpert(), argv[0]);
        castice.hledejZpet();
      }
      else {
        try {
          Castice castice = new Castice(new GUIExpert(), argv[0]);
          castice.hledejZpet();
        } catch(java.lang.InternalError e) {
          //Nepodarilo se spustit graficky, zkusi textove
          Castice  castice = new Castice(new TextUIExpert(), argv[0]);
          castice.hledejZpet();
        }
      }
    }
  }
}

