import java.io.*;
/*
 * Autor: Martin Kot
 * Nazev: Castice
 * Studijni cislo: KOT119
 */

/**
 * Graficka verze rozhrani pro expertni system
 *
 * @author Martin Kot
 * @version 1.00
 */
class GUIExpert implements UIExpert{
  /**Interval, ve kterem je testovan stisk tlacitka v okne*/
  protected long SLEEP_TIME = 200;
  /**Okno zobrazujici prvky grafickeho rozhrani*/
  protected QuestionFrame qFrame;
  /**Identifikator, zda jde o applet (hodnota true) nebo aplikaci (false)*/
  protected boolean applet=false;
  /**Semafor. Hodnota true znamena, ze trida ceka na akci uzivatele v okne*/
  protected boolean stopped = false;
  /**Odkaz na hlavni tridu expertniho systemu*/
  protected Castice castice=null;

  /**Konstruktor*/
  public GUIExpert() {
    qFrame = new QuestionFrame(this);
    qFrame.show();
  }
  /**
   * Konstruktor
   *
   * @param applet Hodnota urcujici, zda jde o applet (true) nebo 
   *               aplikaci (false)
   * @param castice Hlavni trida expertniho systemu
   */
  public GUIExpert(boolean applet,Castice castice) {
    this.applet = applet;
    this.castice = castice;
    qFrame = new QuestionFrame(this);
    qFrame.show();
  }
  
  /**
   * Nastavi hodnotu promenne stopped. Slouzi pro indikaci akce uzivatele v 
   * okne rozhrani.
   *
   * @param stopped Hodnota false oznamuje tride, ze nastal uzivatelsky vstup
   */
  public void setStopped(boolean stopped){
    this.stopped = stopped;
  }

  /**
   * Vola se pri novem hledani cilu
   */
  public void znovu() {
    qFrame.znovu();
  }

  /**
   * Ukonci program
   */
  public void konec() {
    qFrame.dispose();
    qFrame = null;
    stopped = false;
    if(!applet){
      System.exit(0);
    }
    else {
      castice.destroy();
    }
  }

  /**
   * Dotaze se uzivatele na nejakou skutecnost
   *
   * @param text Zneni dotazu
   * @return Hodnota zadana uzivatelem na dotaz ulozena pomoci tridy PravdHod 
   *         umoznujici hodnoty TRUE, FALSE, UNKNOWN
   */
  public PravdHod dotaz(String text) {
    if (qFrame != null){
      qFrame.show();
      qFrame.dotaz(text);
      stopped = true;
      while(stopped){
        try {
          Thread.sleep(SLEEP_TIME);
        } catch (InterruptedException e) {
        }
      }
      if(qFrame != null){
        return qFrame.getDotaz();
      } 
    }
    return new PravdHod(PravdHod.FALSE);
  }


  /**
   * Zjisti, zda chce uzivatel pokracivat urcovanim novych cilu
   *
   * @return true, pokud uzivatel zvoli moznost pokracovani, false jinak
   */
  public boolean pokracovat() {
    if(qFrame == null) { 
      return false;
    }
    boolean retVal;
    qFrame.pokracovat();
    stopped = true;
    while(stopped){
      try {
        Thread.sleep(SLEEP_TIME);
      } catch (InterruptedException e) {
      }
    }

    retVal = (qFrame.getDotaz().getVal() == PravdHod.TRUE)?true:false;
    if(retVal==false){
      konec();
    }
    else {
      qFrame.dotazy();
    }
    return retVal;
  }

  /**
   * Zobrazi hlaseni o nenalezeni zadneho platneho cile
   */
  public void nenasel() {
    if(qFrame != null) {
      qFrame.nenasel();
    }
  }
  
  /**
   * Zobrazi platny cil
   *
   * @param text Text, ktery je zobrazen jako cil
   */
  public void cil(String text) {
    if(qFrame != null) {
      qFrame.cil(text);
    }
  }
  
  /**
   * Zobrazi nepotrvrzeny, ale ani nevyvraceny cil
   *
   * @param text Text, ktery je zobrazen jako cil
   */
  public void cilUnknown(String text) {
    if(qFrame != null) {
      qFrame.cilUnknown(text);
    }
  }
}
